@@ -23,6 +23,7 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 #include "support/mutex.h"
+#include "support/docstream.h"
 #include "support/os.h"
 #include "support/PathChanger.h"
 #include "support/Systemcall.h"
@@ -33,7 +34,7 @@
 #include <map>
 #include <ctime>
 
-// FIXME: Q_WS_MACX is not available, it's in Qt
+// FIXME: Q_OS_MAC is not available, it's in Qt
 #ifdef USE_MACOSX_PACKAGING
 #include "support/linkback/LinkBackProxy.h"
 #endif
@@ -101,20 +102,22 @@ private:
 };
 
 
-} //namespace anon
+} // namespace
 
 bool Format::formatSorter(Format const * lhs, Format const * rhs)
 {
-	return _(lhs->prettyname()) < _(rhs->prettyname());
+	return compare_locale(translateIfPossible(lhs->prettyname()),
+	                      translateIfPossible(rhs->prettyname())) < 0;
 }
 
 bool operator<(Format const & a, Format const & b)
 {
-	return _(a.prettyname()) < _(b.prettyname());
+	return compare_locale(translateIfPossible(a.prettyname()),
+	                      translateIfPossible(b.prettyname())) < 0;
 }
 
 
-Format::Format(string const & n, string const & e, string const & p,
+Format::Format(string const & n, string const & e, docstring const & p,
 	       string const & s, string const & v, string const & ed,
 	       string const & m, int flags)
 	: name_(n), prettyname_(p), shortcut_(s), viewer_(v),
@@ -169,9 +172,9 @@ void Format::setExtensions(string const & e)
 Format const * Formats::getFormat(string const & name) const
 {
 	FormatList::const_iterator cit =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (cit != formatlist.end())
+	if (cit != formatlist_.end())
 		return &(*cit);
 	else
 		return 0;
@@ -229,26 +232,26 @@ string guessFormatFromContents(FileName const & fn)
 	static string const zipStamp = "PK";
 
 	// ZIP containers (koffice, openoffice.org etc).
-	static string const nonzipStamp = "\008\0\0\0mimetypeapplication/";
+	static string const nonzipStamp = "\010\0\0\0mimetypeapplication/";
 
 	// compress
 	static string const compressStamp = "\037\235";
 
+	// DOS binary EPS according to Adobe TN-5002
+	static string const binEPSStamp = "\xC5\xD0\xD3\xC6";
+
+
 	// Maximum strings to read
 	int const max_count = 50;
 	int count = 0;
 
-	// Maximum number of binary chars allowed for latex detection
-	int const max_bin = 5;
-
 	string str;
 	string format;
 	bool firstLine = true;
 	bool backslash = false;
 	bool maybelatex = false;
-	int binchars = 0;
 	int dollars = 0;
-	while ((count++ < max_count) && format.empty() && binchars <= max_bin) {
+	while ((count++ < max_count) && format.empty() && !maybelatex) {
 		if (ifs.eof())
 			break;
 
@@ -278,6 +281,8 @@ string guessFormatFromContents(FileName const & fn)
 
 			} else if (stamp == "\001\332") {
 				format =  "sgi";
+			} else if (prefixIs(str, binEPSStamp)) {
+				format =  "eps";
 
 			// PBM family
 			// Don't need to use str.at(0), str.at(1) because
@@ -373,17 +378,9 @@ string guessFormatFromContents(FileName const & fn)
 				// inline equation
 				maybelatex = true;
 		}
-
-		// Note that this is formally not correct, since count_bin_chars
-		// expects utf8, and str can be anything: plain text in any
-		// encoding, or really binary data. In practice it works, since
-		// QString::fromUtf8() drops invalid utf8 sequences, and while
-		// the exact number may not be correct, we still get a high
-		// number for truly binary files.
-		binchars += count_bin_chars(str);
 	}
 
-	if (format.empty() && binchars <= max_bin && maybelatex)
+	if (format.empty() && maybelatex && !isBinaryFile(fn))
 		format = "latex";
 
 	if (format.empty()) {
@@ -400,7 +397,7 @@ string guessFormatFromContents(FileName const & fn)
 	return string();
 }
 
-}
+} // namespace
 
 
 string Formats::getFormatFromFile(FileName const & filename) const
@@ -408,6 +405,7 @@ string Formats::getFormatFromFile(FileName const & filename) const
 	if (filename.empty())
 		return string();
 
+	string psformat;
 	string format;
 #ifdef HAVE_MAGIC_H
 	if (filename.exists()) {
@@ -428,16 +426,21 @@ string Formats::getFormatFromFile(FileName const & filename) const
 						<< "\tCouldn't query magic database - "
 						<< magic_error(magic_cookie));
 				}
-				// we need our own ps/eps detection
-				if (!mime.empty() && mime != "application/postscript" &&
+				// our own detection is better for binary files (can be anything)
+				// and different plain text formats
+				if (!mime.empty() && mime != "application/octet-stream" &&
 				    mime != "text/plain") {
 					Formats::const_iterator cit =
-						find_if(formatlist.begin(), formatlist.end(),
+						find_if(formatlist_.begin(), formatlist_.end(),
 							FormatMimeEqual(mime));
-					if (cit != formats.end()) {
+					if (cit != formatlist_.end()) {
 						LYXERR(Debug::GRAPHICS, "\tgot format from MIME type: "
 							<< mime << " -> " << cit->name());
-						format = cit->name();
+						// See special eps/ps handling below
+						if (mime == "application/postscript")
+							psformat = cit->name();
+						else
+							format = cit->name();
 					}
 				}
 			}
@@ -453,11 +456,22 @@ string Formats::getFormatFromFile(FileName const & filename) const
 
 	string const ext = getExtension(filename.absFileName());
 	if (format.empty()) {
-		string const format = guessFormatFromContents(filename);
+		// libmagic does not distinguish eps and ps.
+		// Therefore we need to use our own detection here, but only if it
+		// recognizes either ps or eps. Otherwise the libmagic guess will
+		// be better (bug 9146).
+		format = guessFormatFromContents(filename);
+		if (!psformat.empty()) {
+			if (isPostScriptFileFormat(format))
+				return format;
+			else
+				return psformat;
+		}
+
 		if (isZippedFileFormat(format) && !ext.empty()) {
-			string const & fmt_name = formats.getFormatFromExtension(ext);
+			string const & fmt_name = getFormatFromExtension(ext);
 			if (!fmt_name.empty()) {
-				Format const * p_format = formats.getFormat(fmt_name);
+				Format const * p_format = getFormat(fmt_name);
 				if (p_format && p_format->zippedNative())
 					return p_format->name();
 			}
@@ -486,9 +500,9 @@ string Formats::getFormatFromExtension(string const & ext) const
 		// this is ambigous if two formats have the same extension,
 		// but better than nothing
 		Formats::const_iterator cit =
-			find_if(formatlist.begin(), formatlist.end(),
+			find_if(formatlist_.begin(), formatlist_.end(),
 				FormatExtensionsEqual(ext));
-		if (cit != formats.end()) {
+		if (cit != formatlist_.end()) {
 			LYXERR(Debug::GRAPHICS, "\twill guess format from file extension: "
 				<< ext << " -> " << cit->name());
 			return cit->name();
@@ -519,6 +533,10 @@ bool Formats::isZippedFile(support::FileName const & filename) const {
 	map<string, ZippedInfo>::iterator it = zipped_.find(fname);
 	if (it != zipped_.end() && it->second.timestamp == timestamp)
 		return it->second.zipped;
+	// FIXME perf: This very expensive function is called on startup on each
+	// file whic is going to be parsed, and also on svgz icons. Maybe there is a
+	// quicker way to check whether a file is zipped?  I.e. for icons we
+	// probably just need to check the extension (svgz vs svg).
 	string const & format = getFormatFromFile(filename);
 	bool zipped = (format == "gzip" || format == "zip");
 	zipped_.insert(make_pair(fname, ZippedInfo(zipped, timestamp)));
@@ -559,8 +577,8 @@ static string fixCommand(string const & cmd, string const & ext,
 
 void Formats::setAutoOpen()
 {
-	FormatList::iterator fit = formatlist.begin();
-	FormatList::iterator const fend = formatlist.end();
+	FormatList::iterator fit = formatlist_.begin();
+	FormatList::iterator const fend = formatlist_.end();
 	for ( ; fit != fend ; ++fit) {
 		fit->setViewer(fixCommand(fit->viewer(), fit->extension(), os::VIEW));
 		fit->setEditor(fixCommand(fit->editor(), fit->extension(), os::EDIT));
@@ -571,10 +589,10 @@ void Formats::setAutoOpen()
 int Formats::getNumber(string const & name) const
 {
 	FormatList::const_iterator cit =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (cit != formatlist.end())
-		return distance(formatlist.begin(), cit);
+	if (cit != formatlist_.end())
+		return distance(formatlist_.begin(), cit);
 	else
 		return -1;
 }
@@ -583,21 +601,21 @@ int Formats::getNumber(string const & name) const
 void Formats::add(string const & name)
 {
 	if (!getFormat(name))
-		add(name, name, name, string(), string(), string(),
+		add(name, name, from_utf8(name), string(), string(), string(),
 		    string(), Format::document);
 }
 
 
 void Formats::add(string const & name, string const & extensions,
-		  string const & prettyname, string const & shortcut,
+		  docstring const & prettyname, string const & shortcut,
 		  string const & viewer, string const & editor,
 		  string const & mime, int flags)
 {
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it == formatlist.end())
-		formatlist.push_back(Format(name, extensions, prettyname,
+	if (it == formatlist_.end())
+		formatlist_.push_back(Format(name, extensions, prettyname,
 					    shortcut, viewer, editor, mime, flags));
 	else
 		*it = Format(name, extensions, prettyname, shortcut, viewer,
@@ -608,16 +626,16 @@ void Formats::add(string const & name, string const & extensions,
 void Formats::erase(string const & name)
 {
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
-		formatlist.erase(it);
+	if (it != formatlist_.end())
+		formatlist_.erase(it);
 }
 
 
 void Formats::sort()
 {
-	std::sort(formatlist.begin(), formatlist.end());
+	std::sort(formatlist_.begin(), formatlist_.end());
 }
 
 
@@ -625,9 +643,9 @@ void Formats::setViewer(string const & name, string const & command)
 {
 	add(name);
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
+	if (it != formatlist_.end())
 		it->setViewer(command);
 }
 
@@ -636,9 +654,9 @@ void Formats::setEditor(string const & name, string const & command)
 {
 	add(name);
 	FormatList::iterator it =
-		find_if(formatlist.begin(), formatlist.end(),
+		find_if(formatlist_.begin(), formatlist_.end(),
 			FormatNamesEqual(name));
-	if (it != formatlist.end())
+	if (it != formatlist_.end())
 		it->setEditor(command);
 }
 
@@ -662,7 +680,7 @@ bool Formats::view(Buffer const & buffer, FileName const & filename,
 // by the caller (this should be "utility" code)
 		Alert::error(_("Cannot view file"),
 			bformat(_("No information for viewing %1$s"),
-				prettyName(format_name)));
+				translateIfPossible(prettyName(format_name))));
 		return false;
 	}
 	// viewer is 'auto'
@@ -677,7 +695,29 @@ bool Formats::view(Buffer const & buffer, FileName const & filename,
 		}
 	}
 
-	string command = libScriptSearch(format->viewer());
+	string command = format->viewer();
+
+	// Escape backslashes if not already in double or single quotes.
+	// We cannot simply quote the whole command as there may be arguments.
+	if (contains(command, '\\')) {
+		bool inquote1 = false;
+		bool inquote2 = false;
+		string::iterator cit = command.begin();
+		for (; cit != command.end(); ++cit) {
+			switch (*cit) {
+			case '"':
+				inquote1 = !inquote1;
+				break;
+			case '\'':
+				inquote2 = !inquote2;
+				break;
+			case '\\':
+				if (!inquote1 && !inquote2)
+					cit = ++command.insert(cit, '\\');
+				break;
+			}
+		}
+	}
 
 	if (format_name == "dvi" &&
 	    !lyxrc.view_dvi_paper_option.empty()) {
@@ -694,8 +734,10 @@ bool Formats::view(Buffer const & buffer, FileName const & filename,
 	if (!contains(command, token_from_format))
 		command += ' ' + token_from_format;
 
-	command = subst(command, token_from_format, quoteName(onlyFileName(filename.toFilesystemEncoding())));
-	command = subst(command, token_path_format, quoteName(onlyPath(filename.toFilesystemEncoding())));
+	command = subst(command, token_from_format,
+	                quoteName(onlyFileName(filename.toFilesystemEncoding()), quote_shell_filename));
+	command = subst(command, token_path_format,
+	                quoteName(onlyPath(filename.toFilesystemEncoding()), quote_shell_filename));
 	command = subst(command, token_socket_format, quoteName(theServerSocket().address()));
 	LYXERR(Debug::FILES, "Executing command: " << command);
 	// FIXME UNICODE utf8 can be wrong for files
@@ -703,7 +745,8 @@ bool Formats::view(Buffer const & buffer, FileName const & filename,
 
 	PathChanger p(filename.onlyPath());
 	Systemcall one;
-	one.startscript(Systemcall::DontWait, command, buffer.filePath());
+	one.startscript(Systemcall::DontWait, command,
+	                buffer.filePath(), buffer.layoutPos());
 
 	// we can't report any sort of error, since we aren't waiting
 	return true;
@@ -741,7 +784,7 @@ bool Formats::edit(Buffer const & buffer, FileName const & filename,
 // be done by the caller (this should be "utility" code)
 		Alert::error(_("Cannot edit file"),
 			bformat(_("No information for editing %1$s"),
-				prettyName(format_name)));
+				translateIfPossible(prettyName(format_name))));
 		return false;
 	}
 
@@ -762,15 +805,18 @@ bool Formats::edit(Buffer const & buffer, FileName const & filename,
 	if (!contains(command, token_from_format))
 		command += ' ' + token_from_format;
 
-	command = subst(command, token_from_format, quoteName(filename.toFilesystemEncoding()));
-	command = subst(command, token_path_format, quoteName(onlyPath(filename.toFilesystemEncoding())));
+	command = subst(command, token_from_format,
+	                quoteName(filename.toFilesystemEncoding(), quote_shell_filename));
+	command = subst(command, token_path_format,
+	                quoteName(onlyPath(filename.toFilesystemEncoding()), quote_shell_filename));
 	command = subst(command, token_socket_format, quoteName(theServerSocket().address()));
 	LYXERR(Debug::FILES, "Executing command: " << command);
 	// FIXME UNICODE utf8 can be wrong for files
 	buffer.message(_("Executing command: ") + from_utf8(command));
 
 	Systemcall one;
-	one.startscript(Systemcall::DontWait, command, buffer.filePath());
+	one.startscript(Systemcall::DontWait, command,
+	                buffer.filePath(), buffer.layoutPos());
 
 	// we can't report any sort of error, since we aren't waiting
 	return true;
@@ -781,7 +827,7 @@ docstring const Formats::prettyName(string const & name) const
 {
 	Format const * format = getFormat(name);
 	if (format)
-		return from_utf8(format->prettyname());
+		return format->prettyname();
 	else
 		return from_utf8(name);
 }
@@ -822,17 +868,18 @@ FlavorTranslator initFlavorTranslator()
 	f.addPair(OutputParams::XML, "docbook-xml");
 	f.addPair(OutputParams::HTML, "xhtml");
 	f.addPair(OutputParams::TEXT, "text");
+	f.addPair(OutputParams::LYX, "lyx");
 	return f;
 }
 
 
 FlavorTranslator const & flavorTranslator()
 {
-	static FlavorTranslator translator = initFlavorTranslator();
+	static FlavorTranslator const translator = initFlavorTranslator();
 	return translator;
 }
 
-}
+} // namespace
 
 
 std::string flavor2format(OutputParams::FLAVOR flavor)
@@ -847,9 +894,4 @@ OutputParams::FLAVOR format2flavor(std::string fmt)
 	return flavorTranslator().find(fmt);
 } */
 
-Formats formats;
-
-Formats system_formats;
-
-
 } // namespace lyx
